@@ -1,6 +1,3 @@
-use serde::de::DeserializeOwned;
-use serde::Serialize;
-
 use crate::checkout::Checkout;
 use crate::context::Context;
 use crate::error::new_application_error;
@@ -16,11 +13,20 @@ use super::LookupKey;
 use super::Order;
 use super::OrderState;
 
-pub async fn confirm_handler<C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
+pub async fn confirm_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
-) -> Result<Order<P>, Error> {
+) -> Result<
+    Order<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_od = ctx.get_order_for_update(&order_id).await?;
     if maybe_od.is_none() {
         return Err(new_not_found_error(
@@ -30,15 +36,25 @@ pub async fn confirm_handler<C: Context + Send, P: Sync + Send + Serialize + Des
 
     let mut od = maybe_od.unwrap();
     od.confirm(internal_ctx, ctx).await?;
+    od.populate_associations(ctx).await?;
 
     Ok(od)
 }
 
-pub async fn cancel_handler<C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
+pub async fn cancel_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
-) -> Result<Order<P>, Error> {
+) -> Result<
+    Order<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     // first retrieve the associated payment for update to avoid deadlock
     let maybe_payment = get_order_payment_for_update(ctx, order_id).await?;
 
@@ -59,25 +75,29 @@ pub async fn cancel_handler<C: Context + Send, P: Sync + Send + Serialize + Dese
         }
 
         let mut payment = maybe_payment.unwrap();
-        cancel_payment(
-            internal_ctx,
-            ctx,
-            &mut payment,
-            Initiator::<P>::Order(&mut od),
-        )
-        .await?;
+        cancel_payment(internal_ctx, ctx, &mut payment, Initiator::Order(&mut od)).await?;
     } else {
         od.cancel(internal_ctx, ctx).await?;
     }
 
+    od.populate_associations(ctx).await?;
+
     Ok(od)
 }
 
-pub async fn reconcile_handler<C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
+pub async fn reconcile_handler<C: Context + Send>(
     ctx: &mut C,
     lookup_key: LookupKey,
 ) -> Result<(), Error> {
-    let maybe_order: Option<Order<P>> = match &lookup_key {
+    let maybe_order: Option<
+        Order<
+            Payment<
+                <C as PaymentProcessor>::PrivateData,
+                <C as PaymentProcessor>::PublicData,
+                <C as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
+    > = match &lookup_key {
         LookupKey::CheckoutId(checkout_id) => get_order_by_checkout_id(ctx, checkout_id).await?,
         LookupKey::OrderId(order_id) => ctx.get_order_for_update(order_id).await?,
     };
@@ -94,13 +114,21 @@ pub async fn reconcile_handler<C: Context + Send, P: Sync + Send + Serialize + D
     od.reconcile(ctx).await
 }
 
-async fn get_order_by_checkout_id<
-    C: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
+async fn get_order_by_checkout_id<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
-) -> Result<Option<Order<P>>, Error> {
+) -> Result<
+    Option<
+        Order<
+            Payment<
+                <C as PaymentProcessor>::PrivateData,
+                <C as PaymentProcessor>::PublicData,
+                <C as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
+    >,
+    Error,
+> {
     let maybe_checkout: Option<
         Checkout<
             Payment<
