@@ -277,6 +277,17 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
             )),
         }
     }
+
+    pub async fn populate_associations<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+    ) -> Result<(), Error> {
+        if let Some(ref payment_id) = self.payment_id {
+            self.payment = ctx.get_payment(payment_id).await?;
+        }
+
+        Ok(())
+    }
 }
 
 impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
